/*

  Programma Pianeti.ccp:
  integra l'equazione del moto di un pianeta intorno al sole.
  Equazioni:

  d^2 x / dt^2 = - G * x / R^3
  d^2 y / dt^2 = - G * y / R^3

 */

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

#include "RK2.h"


// Variabli globali (parametri dell'equazione, da utilizzare in dydt!)

double G;


/*

  Programma principale

*/

int main()
{

  cout << "Programma che integra l'equazione del moto di un pianeta" << endl;
  cout << "con uno schema di Runge-Kutta al secondo ordine." << endl;

  int Neq = 4;

  cout << "Inserire dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  cout << "Inserire il valore di G: ";
  cin >> G;

  cout << "Inserire le condizioni iniziali x0, vx0, y0, vy0: ";
  double y[Neq];
  cin >> y[0] >> y[1] >> y[2] >> y[3];

  RK2 soluzione = RK2( Neq, y );

  cout << "Nome del file di output: ";
  char nfile[256];
  cin >> nfile;

  double tc = 0.0, tn;

  // Calcolo invarianti: Energia e Momento angolare

  double R  = sqrt( y[0] * y[0] + y[2] * y[2] );
  double E  = 0.5 * ( y[1] * y[1] + y[3] * y[3] ) - G / R;
  double Lz = y[0] * y[3] - y[2] * y[1];

  // Apertura del file di output e scrittura condizioni iniziali

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  fout << "#   t     x    vx    y    vy   E   Lz" << endl;
  fout << tc;
  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
  fout << "   " << E << "   " << Lz;
  fout << endl;

  // Ciclo temporale

  for ( long n = 1; n <= Nend; n++ )
    {

      tc = ( n - 1 ) * dt;

      // Avanza di un passo lo schema

      soluzione.stepRK2( tc, dt );
      tn = n * dt;

      // Stampa ogni Tprint intervalli di stampa

      if ( ( n % Nprint ) == 0 )
	{
	  soluzione.getValue( y );
	  R  = sqrt( y[0] * y[0] + y[2] * y[2] );
	  E  = 0.5 * ( y[1] * y[1] + y[3] * y[3] ) - G / R;
	  Lz = y[0] * y[3] - y[2] * y[1];
	  fout << tn;
	  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
	  fout << "   " << E << "   " << Lz;
	  fout << endl;
	}

    }

  fout.close();

}


// Funzione dydt: calcola il secondo membro dell'equazione

void dydt( int Neq, double t, double * y, double * dy )
{
  double R = sqrt( y[0] * y[0] + y[2] * y[2] );
  double R3 = R * R * R;

  dy[0] = y[1];
  dy[1] = - G * y[0] / R3;
  dy[2] = y[3];
  dy[3] = - G * y[2] / R3;
}
